/*
	Tomasz Piekarczyk
	grupa dziekaska G1EIK
	nr albumu 200732
	temat: zastosowanie biblioteki tribool

	Zadanie programu:
		Program opisuje dziaanie biblioteki boost::tribool. W szczegolnoci tworzenia
		prostych konstrukcji logicznych na algebrze trjstanowej. Dotatkowo pokazany 
		jest sposb modyfikacji nazwy trzeciego stanu oraz korzystanie z operatora '<<'
		przy ustawionej fladze 'boolalpha'.
		
	Zasada dziaania programu:
		Program tworzy tablic sensorow opisujcych stan rnych parametrow fikcyjnej 
		"elektrowni jdrowej". Sensory zwracaj stan parametrw w jednej z trzech 
		wartoci '1' - OK; '0' - poza norm; 'indeterminate' - nie pobrano pomiaru. 
		W przypadku odpowiedniej kombinacji stanw rnych czynnikow ( np. czujnik 
		temp = 0 && czujnik radiacji = indeterminate ) generowane jest ostrzeenie. 
*/	

#include <iostream>
#include <boost/logic/tribool.hpp>
#include <boost/logic/tribool_io.hpp>
#include <map>
#include<string>

BOOST_TRIBOOL_THIRD_STATE(notRead)	//makro umoliwiajce zamian standardowej nazwy stanu
					//nieokrelonego (indeterminate) na wybran przez uytkownika
using namespace boost::logic;
using namespace std;    

locale g_global;
locale g_Locale( g_global, new indeterminate_name<char>("notRead") );	//dodanie do locale nowej nazwy stanu nieokrelonego

/**
*	@brief Klasa reprezentuje sensor ktory kontroluje czy obserwowany parametr ma warto w bezpiecznym zakresie.  
*/
class Sensor
{
	private:
		
		const int m_iUpperLimit;
		const int m_iLowerLimit;
		int m_iCurrentValue;
		tribool m_tbState;
		unsigned short m_ushSamplesUnread;	//liczba prbek nieodczytanych pod rzd
		string m_sResponsibility;
		void GetSample();
	public:

		Sensor(): m_iUpperLimit(0), m_iLowerLimit(0)
		{};
		Sensor( int, int, string );
		tribool GetState();
		string MyResponsibility() const
		{ 
			return m_sResponsibility; 
		};
		unsigned short HowManyUnread() const
		{
			return m_ushSamplesUnread;
		};
};

int main(int argc, char *argv[])
{
	cout.imbue(g_Locale);			//ustawia nowo utworzony locale w ostream
	time_t m_CurrentTime = time( NULL ) ;	
	srand( m_CurrentTime );			//ustawienie ziarna dla rand;	

	const int SENSNUMBER = 5;		//liczba sensorw
	const string PRES = "Pressure"; 
	const string TEMP = "Temperature";
	const string VOLT = "Voltage";
	const string BATT = "Battery level";
	const string RADN = "Radiation level";

	map<string, tribool> l_stateTable;	//tablica hashujca (nazwa sensora,aktualny stan) 
	char l_cContinue;			//warunek zakoczenia programu
	map<string, tribool>::iterator it;	
	
	Sensor l_Sensors[SENSNUMBER] = 	//tablica sensorw	
	{
		Sensor( 500,1000, PRES ),
		Sensor( 0, 80, TEMP ),
		Sensor( 200, 260, VOLT ),
		Sensor( 50, 100, BATT ),
		Sensor( 0, 50, RADN )
	};					
	

	do
	{
		//generacja kolejnego cigu prbek
		for( int i=0; i< SENSNUMBER; i++ )	
			l_stateTable[l_Sensors[i].MyResponsibility()] = l_Sensors[i].GetState();
		
		/*
			Wypisanie caej tablicy hashujcej z wykorzystaniem flagi boolalpha co 
			oznacza e standardowy wypis w postaci '0', '1', '2' zastpiono 
			odpowiednio 'false' 'true' 'indeterminate' (lub jak w naszym przypadku 
			notRead)
		*/
		for ( it=l_stateTable.begin() ; it != l_stateTable.end(); it++ )
    			cout  << (*it).first << " => " << boolalpha << (*it).second << endl;
		cout << endl;
		
		/*
			Kiedy czujnikowi dwukrotnie nie powiodo si odczytanie wartoci (stan 
			notRead) ostrzeenie zostaje wypisane na ekran 
		*/
		for( int i=0; i< SENSNUMBER; i++ )
			if( l_Sensors[i].HowManyUnread() > 1)	
				cout << "UWAGA!!! Czujnik "<< l_Sensors[i].MyResponsibility() << " uszkodzony "<< endl;
		
		/*
			Cig przykadw tworzenia funkcji logicznych z uyciem tribool przy 
			opisach a - 1 argument, b - 2 argument
		*/

		//tylko kiedy a=0 and b=0
		if( !l_stateTable[TEMP] && !l_stateTable[PRES] )		
			cout << "UWAGA!!! Zwiekszone ryzyko przegrzania."<< endl;
		// tylko kiedy a=0
		if( !l_stateTable[VOLT] ) 
			cout << "UWAGA!!! Niestabilne napiecie" << endl;
		//kiedy a=notRead or b=0
		if( notRead( l_stateTable[RADN]) || !l_stateTable[RADN]  )	
			cout << "UWAGA!!! Nieznany badz przekroczony poziom radiacji" << endl;
		//kiedy a=notRead
		if( notRead(l_stateTable[BATT]) )	
			cout << "UWAGA!!! Nieznany stan generatorow" << endl;
		//kiedy a=0 and b=0
		if( !l_stateTable[BATT] && !l_stateTable[TEMP] )	
			cout << "UWAGA!!! Awaryjny system chlodzenia uszkodzony" << endl;
	
		cin.clear();
		cout << endl <<  "Kontynuacja? [t/n]" << endl; 
		cin >> l_cContinue;
		cout << endl;
	}
	//koniec programu jesli uytkownik tak zadecydowa
	while( l_cContinue != 'n');	
	
	return 0;
}

/**
 * @brief Konstrukuuje sensor o zadanych limitach i nadaje nazw sensorowi
 * @param lowLimit Poniej tej wartoci sensor ustawia warto stanu na niepoprawn (0)
 * @param upLimit Powyej tej wartoci sensor ustawia warto stanu na niepoprawn (0)
 * @param resp Nazwa obiektu parametru za ktry odpowiada sensor
 */
Sensor::Sensor( int lowLimit, int upLimit, string resp ): m_iUpperLimit(upLimit), m_iLowerLimit(lowLimit)
{
	m_tbState = true;
	m_sResponsibility = resp;
	m_ushSamplesUnread = 0;
}

/**
 * @brief Pobiera aktualny stan sensora i zwraca stan w jakim znajduje si sensor
 * @return Zwraca '0' jeli sensor wykry przekroczenie wartoci dozwolonej, '1' jeli warto 
 *	   mieci si w zakresie wartoci dozwolonych, 'notRead' jeli nie powid si odczyt danych 
 *	   ( losowe zaburzenia )
 */
tribool Sensor::GetState()
{

	GetSample();
	return m_tbState;
}

/**
 * @brief Generuje kolejn probk danych
 */
void Sensor::GetSample()
{
	//generacja liczby losowej
	m_iCurrentValue =   static_cast<int>(2 * m_iUpperLimit * (static_cast<double>( rand() ) / RAND_MAX ));	
	
	//losowe zaburzenie odczytu czyli stan notRead
	if(  (static_cast<double>(rand()) / RAND_MAX ) > 0.7F )	
	{
		m_tbState = notRead;
		m_ushSamplesUnread++;	
	}
	//czy aktualna probka mieci si w limitach okrelonych dla tego sensora
	else if( m_iCurrentValue >= m_iUpperLimit || m_iCurrentValue <= m_iLowerLimit )	
	{
		m_tbState = false;
		m_ushSamplesUnread = 0;
	}
	else
	{
		m_tbState = true;
		m_ushSamplesUnread = 0;
	}	
}
